//** 24 Jan 2004 
//** Comments and mods by Simone Mannori 
#ifndef __SCICOS_BLOCK_H__
#define __SCICOS_BLOCK_H__

#ifdef __STDC__
#include <stdlib.h>
#else 
#include <malloc.h>
#endif 

typedef void (*voidg)();

typedef struct {
  int nevprt   ; /* binary coding of activation inputs, -1 if internally activated */    
  voidg funpt  ; /* pointer: pointer to the computational function */
  int type     ; /* type id of interfacing function, current type is 4 ( "C" code ) */   
  int scsptr   ; /* not used for C interfacing function */
  
  int nz       ; /* size of discrete time state */
  double *z    ; /* discrete-time state (vector of size nz) */  
  
  int nx       ; /* size of the continous time state       */ 
  double *x;   ; /* continous-time state (array of size nx) */
  double *xd   ; /* derivative of the continous-time state (array of size nx) */
  double *res  ; /* only used for internal implicit blocks (vector of size nx)*/ 
  
  int nin      ; /* number of input (size of the input vector = number of input */    
  int *insz    ; /* input size ( vector that specify the size of each input )*/
                 /* e.g. nin = 4 ; *insz = [2 3 1]  
		    input_1, size 2 
		    input_2, size 3
		    input_3, size 1
		 */ 
  double **inptr ; /* table of pointers to inputs : where you really get the values from */
  
  int   nout    ; /* number of outputs */
  int *outsz    ; /* ouput size (see the explanation for the input) */
  double **outptr;/* table of pointers to output : where you really put the values to */
  
  int nevout;    /* number of activation outputs ports */
  double *evout; /* output activation ports value*/
  
  
  /* the following four fields are useful to parametrize the block */
  int nrpar  ;   /* number of real parameter */ 
  double *rpar;  /* real parameters */
  
  int nipar;  /* number of integer parameters */  
  int *ipar;  /* integer parameters           */
  
  int ng;    /* number of zero crossing surface */
  double *g; /* sero crossing surgface          */
  int ztyp;  /* boolean, true only if block MAY have zero-crossing */
  int *jroot ; /* vector (size ng) indicating the presence and direction of crossing */
  
  char *label ; /* block label */
  void **work ; /* pointer to workspace if allocation done by block */
  
  int nmode   ; /* number of modes */
  int *mode   ; /* mode vector (size nmode) */

} scicos_block;


void do_cold_restart();
int get_phase_simulation();
double get_scicos_time();
int get_block_number();
void set_block_error(int);
void set_pointer_xproperty(int* pointer);
void * scicos_malloc(size_t );
void scicos_free(void *p);
double Get_Jacobian_parameter(void);
double Get_Scicos_SQUR(void);
void Set_Jacobian_flag(int flag);

#if WIN32
#undef min
#undef max
#endif

#define max(a,b) ((a) >= (b) ? (a) : (b))
#define min(a,b) ((a) <= (b) ? (a) : (b))

extern int s_copy();
extern int s_cmp();


#endif /* __SCICOS_BLOCK_H__ */
